#!/usr/bin/env perl
# sam2unseq.pl : pull unplaced large seq subsets
# samtools view -F 0x104 at22vcan2d_chrs_ERR8666125_mim.bam | perl sam2unseq.pl > at22vcan2d_chrs_ERR8666125lm_unplseq.fa

$MINSEQ=$ENV{minseq}||150;

while(<>) { 
  ($rd,$fl,$cr,$cb,$cx,$cig)=@v=split; 
  if($rd ne $lrd) { putv($lrd,$rfa) if($lrd and $rfa); $rfa=""; $rrev=0; @an=(); }
  $fa=$v[9]; $rl=length($fa); $rl=$lrl if($rl<2); 
  $lrl=$rl; $lrd=$rd;
  ($cutb,$tb)= $cig=~m/^(\d+)([SH])/?($1,$2):(0,0);
  ($cute,$te)= $cig=~m/(\d+)([SH])$/?($1,$2):(0,0);
  $aln=0; while($cig=~m/(\d+)([MI])/g){ $aln+=$1; }
  if($fl < 0x100){ $rfa=$fa; $rrev=($fl & 0x10)?1:0; }
  else { $srev=($fl & 0x10)?1:0; ($cutb,$cute)=($cute,$cutb) if($srev ne $rrev); }
  %c=(); $ct=0; while($cig=~m/(\d+)(\D)/g){ $c{$2}+=$1; $ct+=$1 unless($2 eq "D" or $2 eq "H"); } 
  $cs=join",", map{ $c{$_}.$_ } sort keys %c; $cigb="$ct/$rl,".$cs; 
  push @an,[$cutb,$cute,$aln,$cr,$cb,$fl,$cigb];  
} 

END{  putv($lrd,$rfa) if($lrd and $rfa);  $rfa=""; @an=(); }

sub putv{ 
  my($lrd,$rfa)=@_;
  @an= sort{ $a->[0] <=> $b->[0] } @an; 
  my($ui,$uan)=(0,""); my @uan=(); my $rlen=length($rfa);
  for my $an (@an){ 
    my($cutb,$cute,$aln,$cr,$cb,$fl,$cigb)=@$an; 
    substr($rfa,$cutb,$aln) = ("n") x $aln;
    ++$ui; push @uan, "$ui,$rlen/$cutb:$aln:$cute,$cr:$cb";
  }  
  $ui=0; my @unfa= split /nnnnn+/,$rfa; 
  for $unfa (@unfa) { 
    $uan=($ui >= @uan)? $uan[-1] : $uan[$ui]; $ui++;
    my $ulen= length($unfa); next if($ulen<$MINSEQ);
    my $uid= "$lrd.up" . $ui; $uid=~s/\./_/g;
    print ">$uid len=$ulen; ui:$uan;\n$unfa\n";
  }
} 

=item sam2unseq tests

  samtools view -F 0x104 zmays20mgd5_chr_SRR15447420_b4_mim.bam | perl sam2unseq.pl > zmays20mgd5_chr_SRR15447420lm_unplseq.fa
  grep '>' zmays20mgd5_chr_SRR15447420lm_unplseq.fa | wc -l = 1086971
  grep '>' zmays20mgd5_chr_SRR15447420lm_unplseq.fa | sed 's/ len=/   /;' | sort -k2,2nr | head
  >SRR15447414_133328_up1	26641; ui:1,27261/26641:603:17,zmays20ref_Chr10:97606204;
  >SRR15447414_1690021_up1	23944; ui:1,24908/23944:948:16,zmays20ref_Chr10:97606204;
  >SRR15447414_1592411_up2	17885; ui:;
  >SRR15447414_197765_up2	17476; ui:;
  >SRR15447414_449018_up1	17412; ui:1,19864/17412:2436:16,zmays20ref_Chr2:174004692;

  samtools view -F 0x104 at22vcan2d_chrs_ERR8666125_mim.bam | perl sam2unseq.pl > at22vcan2d_chrs_ERR8666125lm_unplseq.fa
  grep '>'  at22vcan2d_chrs_ERR8666125lm_unplseq.fa | wc -l  = 252133
  >ERR8666125_795982_up2	31926; ui:1,32439/4:509:31926,at22vcan2d_Chr4:2480336;
  >ERR8666125_459898_up2	29029; ui:1,30304/1229:46:29029,at22vcan2d_Chr3:25561213;
  >ERR8666125_144060_up1	28763; ui:1,36138/28763:7372:3,at22vcan2d_Chr2:5316064;
  >ERR8666125_636590_up2	27438; ui:1,36488/3:9047:27438,at22vcan2d_Chr2:175353;
  >ERR8666125_173449_up2	27423; ui:2,35508/34262:1246:0,at22vcan2d_Chr1:684785;
  >ERR8666125_365414_up1	27033; ui:1,37384/27033:9949:402,at22vcan2d_Chr2:4251441;
  >ERR8666125_502710_up1	25729; ui:1,29596/25729:3527:340,at22vcan2d_Chr2:5316064;
  >ERR8666125_611941_up1	25520; ui:1,26566/25520:569:477,ChrPltd:131440;
  >ERR8666125_531355_up1	25030; ui:1,37358/25030:10031:2297,at22vcan2d_Chr2:5324702;
  >ERR8666125_799256_up2	24798; ui:1,26685/0:1887:24798,at22vcan2d_Chr2:2473955;

grep '>'  at22vcan2d_chrs_ERR8666125lm_unplseq.fa | sed 's/ len=/    /;' | sort -k2,2nr | head -5000 | sed 's/,/      /g;' | cut -f4 | sed 's/:.*//;' | sort | uniq -c | head
    389 at22vcan2d_Chr1
   2664 at22vcan2d_Chr2
    160 at22vcan2d_Chr3
   1052 at22vcan2d_Chr4
    680 at22vcan2d_Chr5
     27 ChrMT
     28 ChrPltd

samtools view -c -f 0x4 at22vcan2d_chrs_ERR8666125unpl_mim.bam   = 175093 / 252133, 69% un-remapped
samtools view -c -F 0x104 at22vcan2d_chrs_ERR8666125unpl_mim.bam = 97791

   
=cut

